package cz.fidentis.analyst.feature.services;

import cz.fidentis.analyst.feature.FeaturePoint;
import cz.fidentis.analyst.feature.provider.FeaturePointTypeProvider;
import cz.fidentis.analyst.feature.services.FeaturePointCsvLoader;
import cz.fidentis.analyst.feature.services.FeaturePointFpLoader;
import java.io.File;
import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;
import static org.junit.jupiter.api.Assertions.assertTrue;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.DisplayName;

/**
 *
 * @author Jakub Kolman
 */
public class FeaturePointImportServiceTest {

    private static final Path testFileDirectory = Paths.get("src", "test", "resources", "cz", "fidentis", "analyst", "feature", "services");
    private static final String TEST_CSV_FILE = "00007_01_landmarks.csv";
    private static final String TEST_FP_FILE = "fp_head1_test.fp";

    @DisplayName("Test if CSV file can be read")
    @Test()
    void fileExistsTest() throws IOException {
        File testCsv = new File(testFileDirectory.toFile(), TEST_CSV_FILE);
        assertTrue(testCsv.canRead());
    }

    @DisplayName("Test loading a CSV file")
    @Test
    void importFeaturePointsCSVTest() throws IOException {
        List<FeaturePoint> featurePoints = FeaturePointCsvLoader.loadFeaturePoints(testFileDirectory.toString(), TEST_CSV_FILE);
        assertTrue(featurePoints != null);
        assertTrue(featurePoints.size() > 0);
        assertTrue(featurePoints.get(0).getX() == -45.3298
                && featurePoints.get(0).getY() == 37.1466
                && featurePoints.get(0).getZ() == -40.5415
                && featurePoints.get(0).getFeaturePointType().equals(
                        FeaturePointTypeProvider.getInstance().getFeaturePointTypeByCode("EX_R"))
        );
    }

    @DisplayName("Test loading a FP file")
    @Test
    void importFeaturePointsTest() throws IOException {
        List<FeaturePoint> featurePoints = FeaturePointFpLoader.loadFeaturePoints(testFileDirectory.toString(), TEST_FP_FILE);
        assertTrue(featurePoints != null);
        assertTrue(featurePoints.size() > 0);
        assertTrue(featurePoints.get(0).getX() == -40.429775
                && featurePoints.get(0).getY() == 45.756405
                && featurePoints.get(0).getZ() == 37.12462
                && featurePoints.get(0).getFeaturePointType().equals(
                        FeaturePointTypeProvider.getInstance().getFeaturePointTypeByCode("EX_R"))
        );
    }

    @DisplayName("Test loading a FP file from generic service")
    @Test
    void importFeaturePointsFpFromGenericServiceTest() throws IOException {
        List<FeaturePoint> featurePoints = FeaturePointImportService.importFeaturePoints(testFileDirectory.toString(), TEST_FP_FILE);
        assertTrue(featurePoints != null);
        assertTrue(featurePoints.size() > 0);
        assertTrue(featurePoints.get(0).getX() == -40.429775
                && featurePoints.get(0).getY() == 45.756405
                && featurePoints.get(0).getZ() == 37.12462
                && featurePoints.get(0).getFeaturePointType().equals(
                        FeaturePointTypeProvider.getInstance().getFeaturePointTypeByCode("EX_R"))
        );
    }

    @DisplayName("Test loading a CSV file from generic service")
    @Test
    void importFeaturePointsCsvFromGenericServiceTest() throws IOException {
        List<FeaturePoint> featurePoints = FeaturePointImportService.importFeaturePoints(testFileDirectory.toString(), TEST_CSV_FILE);
        assertTrue(featurePoints != null);
        assertTrue(featurePoints.size() > 0);
        assertTrue(featurePoints.get(0).getX() == -45.3298
                && featurePoints.get(0).getY() == 37.1466
                && featurePoints.get(0).getZ() == -40.5415
                && featurePoints.get(0).getFeaturePointType().equals(
                        FeaturePointTypeProvider.getInstance().getFeaturePointTypeByCode("EX_R"))
        );
    }
}
